#ifndef _GPIO_REG_REG_H_
#define _GPIO_REG_REG_H_

#define GPIO_PIN10_ADDRESS                       0x00000050
#define GPIO_PIN11_ADDRESS                       0x00000054
#define GPIO_PIN12_ADDRESS                       0x00000058
#define GPIO_PIN13_ADDRESS                       0x0000005c

#endif /* _GPIO_REG_H_ */
