#define CONFIG_APCI_035  1
#define CONFIG_APCI_1032 1
#define CONFIG_APCI_1500 1
#define CONFIG_APCI_1516 1
#define CONFIG_APCI_1564 1
#define CONFIG_APCI_16XX 1
#define CONFIG_APCI_1710 1
#define CONFIG_APCI_2016 1
#define CONFIG_APCI_2032 1
#define CONFIG_APCI_2200 1
#define CONFIG_APCI_3001 1
#define CONFIG_APCI_3120 1
#define CONFIG_APCI_3200 1
#define CONFIG_APCI_3300 1
#define CONFIG_APCI_3501 1
#define CONFIG_APCI_3XXX 1

#include "addi-data/addi_common.c"
