/*
 * Copyright (C) 1999-2010, Broadcom Corporation
 * 
 *      Unless you and Broadcom execute a separate written software license
 * agreement governing use of this software, this software is licensed to you
 * under the terms of the GNU General Public License version 2 (the "GPL"),
 * available at http://www.broadcom.com/licenses/GPLv2.php, with the
 * following added to such license:
 * 
 *      As a special exception, the copyright holders of this software give you
 * permission to link this software with independent modules, and to copy and
 * distribute the resulting executable under terms of your choice, provided that
 * you also meet, for each linked independent module, the terms and conditions of
 * the license of that module.  An independent module is a module which is not
 * derived from this software.  The special exception does not apply to any
 * modifications of the software.
 * 
 *      Notwithstanding the above, under no circumstances may you combine this
 * software in any way with any other Broadcom software provided under a license
 * other than the GPL, without Broadcom's express prior written consent.
 *
 * $Id: epivers.h.in,v 13.25 2005/10/28 18:35:33 Exp $
 *
*/


#ifndef _epivers_h_
#define _epivers_h_

#define	EPI_MAJOR_VERSION	4

#define	EPI_MINOR_VERSION	218

#define	EPI_RC_NUMBER		248

#define	EPI_INCREMENTAL_NUMBER	23

#define	EPI_BUILD_NUMBER	0

#define	EPI_VERSION		4, 218, 248, 23

#define	EPI_VERSION_NUM		0x04daf817


#define	EPI_VERSION_STR		"4.218.248.23"
#define	EPI_ROUTER_VERSION_STR	"4.219.248.23"

#endif 
