/* mt9v011 sensor
 *
 * Copyright (C) 2011 Hans Verkuil <hverkuil@xs4all.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __MT9V011_H__
#define __MT9V011_H__

struct mt9v011_platform_data {
	unsigned xtal;	/* Hz */
};

#endif
