#ifndef __RTL8712_REGDEF_H__
#define __RTL8712_REGDEF_H__

#include "rtl8712_syscfg_regdef.h"
#include "rtl8712_cmdctrl_regdef.h"
#include "rtl8712_macsetting_regdef.h"
#include "rtl8712_timectrl_regdef.h"
#include "rtl8712_fifoctrl_regdef.h"
#include "rtl8712_ratectrl_regdef.h"
#include "rtl8712_edcasetting_regdef.h"
#include "rtl8712_wmac_regdef.h"
#include "rtl8712_powersave_regdef.h"
#include "rtl8712_gp_regdef.h"
#include "rtl8712_debugctrl_regdef.h"

#define HIMR	(RTL8712_INTERRUPT_ + 0x08)

#endif /* __RTL8712_REGDEF_H__*/

